@@ -1,21 +1,7 @@
 // arg_util.rs
 
-// *************************************************************************
-// * Copyright (C) 2019-2020 Daniel Mueller (deso@posteo.net)              *
-// *                                                                       *
-// * This program is free software: you can redistribute it and/or modify  *
-// * it under the terms of the GNU General Public License as published by  *
-// * the Free Software Foundation, either version 3 of the License, or     *
-// * (at your option) any later version.                                   *
-// *                                                                       *
-// * This program is distributed in the hope that it will be useful,       *
-// * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-// * GNU General Public License for more details.                          *
-// *                                                                       *
-// * You should have received a copy of the GNU General Public License     *
-// * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
-// *************************************************************************
+// Copyright (C) 2019-2020 The Nitrocli Developers
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 macro_rules! count {
   ($head:ident) => { 1 };
@@ -49,7 +35,7 @@ macro_rules! Command {
     impl $name {
       pub fn execute(
         self,
-        ctx: &mut crate::ExecCtx<'_>,
+        ctx: &mut crate::Context<'_>,
       ) -> anyhow::Result<()> {
         match self {
           $(
