@@ -1,25 +1,12 @@
 // commands.rs
 
-// *************************************************************************
-// * Copyright (C) 2018-2020 Daniel Mueller (deso@posteo.net)              *
-// *                                                                       *
-// * This program is free software: you can redistribute it and/or modify  *
-// * it under the terms of the GNU General Public License as published by  *
-// * the Free Software Foundation, either version 3 of the License, or     *
-// * (at your option) any later version.                                   *
-// *                                                                       *
-// * This program is distributed in the hope that it will be useful,       *
-// * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-// * GNU General Public License for more details.                          *
-// *                                                                       *
-// * You should have received a copy of the GNU General Public License     *
-// * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
-// *************************************************************************
+// Copyright (C) 2018-2020 The Nitrocli Developers
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 use std::convert::TryFrom as _;
 use std::fmt;
 use std::mem;
+use std::ops::Deref as _;
 use std::thread;
 use std::time;
 use std::u8;
@@ -34,12 +21,13 @@ use nitrokey::GenerateOtp;
 use nitrokey::GetPasswordSafe;
 
 use crate::args;
+use crate::config;
 use crate::pinentry;
-use crate::ExecCtx;
+use crate::Context;
 
 /// Set `libnitrokey`'s log level based on the execution context's verbosity.
-fn set_log_level(ctx: &mut ExecCtx<'_>) {
-  let log_lvl = match ctx.verbosity {
+fn set_log_level(ctx: &mut Context<'_>) {
+  let log_lvl = match ctx.config.verbosity {
     // The error log level is what libnitrokey uses by default. As such,
     // there is no harm in us setting that as well when the user did not
     // ask for higher verbosity.
@@ -53,53 +41,120 @@ fn set_log_level(ctx: &mut ExecCtx<'_>) {
   nitrokey::set_log_level(log_lvl);
 }
 
+/// Create a filter string from the program configuration.
+fn format_filter(config: &config::Config) -> String {
+  let mut filters = Vec::new();
+  if let Some(model) = config.model {
+    filters.push(format!("model={}", model.as_ref()));
+  }
+  if !config.serial_numbers.is_empty() {
+    let serial_numbers = config
+      .serial_numbers
+      .iter()
+      .map(ToString::to_string)
+      .collect::<Vec<_>>();
+    filters.push(format!("serial number in [{}]", serial_numbers.join(", ")));
+  }
+  if let Some(path) = &config.usb_path {
+    filters.push(format!("usb path={}", path));
+  }
+  if filters.is_empty() {
+    String::new()
+  } else {
+    format!(" (filter: {})", filters.join(", "))
+  }
+}
+
+/// Find a Nitrokey device that matches the given requirements
+fn find_device(config: &config::Config) -> anyhow::Result<nitrokey::DeviceInfo> {
+  let devices = nitrokey::list_devices().context("Failed to enumerate Nitrokey devices")?;
+  let nkmodel = config.model.map(nitrokey::Model::from);
+  let mut iter = devices
+    .into_iter()
+    .filter(|device| nkmodel.is_none() || device.model == nkmodel)
+    .filter(|device| {
+      config.serial_numbers.is_empty()
+        || device
+          .serial_number
+          .map(|sn| config.serial_numbers.contains(&sn))
+          .unwrap_or_default()
+    })
+    .filter(|device| config.usb_path.is_none() || config.usb_path.as_ref() == Some(&device.path));
+
+  let device = iter
+    .next()
+    .with_context(|| format!("Nitrokey device not found{}", format_filter(config)))?;
+
+  anyhow::ensure!(
+    iter.next().is_none(),
+    "Multiple Nitrokey devices found{}.  Use the --model, --serial-number, and --usb-path options \
+    to select one",
+    format_filter(config)
+  );
+  Ok(device)
+}
+
+/// Connect to a Nitrokey device that matches the given requirements
+fn connect<'mgr>(
+  manager: &'mgr mut nitrokey::Manager,
+  config: &config::Config,
+) -> anyhow::Result<nitrokey::DeviceWrapper<'mgr>> {
+  let device_info = find_device(config)?;
+  manager
+    .connect_path(device_info.path.deref())
+    .with_context(|| {
+      format!(
+        "Failed to connect to Nitrokey device at path {}",
+        device_info.path
+      )
+    })
+}
+
 /// Connect to any Nitrokey device and do something with it.
-fn with_device<F>(ctx: &mut ExecCtx<'_>, op: F) -> anyhow::Result<()>
+fn with_device<F>(ctx: &mut Context<'_>, op: F) -> anyhow::Result<()>
 where
-  F: FnOnce(&mut ExecCtx<'_>, nitrokey::DeviceWrapper<'_>) -> anyhow::Result<()>,
+  F: FnOnce(&mut Context<'_>, nitrokey::DeviceWrapper<'_>) -> anyhow::Result<()>,
 {
   let mut manager =
     nitrokey::take().context("Failed to acquire access to Nitrokey device manager")?;
 
   set_log_level(ctx);
 
-  let device = match ctx.model {
-    Some(model) => manager.connect_model(model.into()).with_context(|| {
-      anyhow::anyhow!("Nitrokey {} device not found", model.as_user_facing_str())
-    })?,
-    None => manager.connect().context("Nitrokey device not found")?,
-  };
-
+  let device = connect(&mut manager, &ctx.config)?;
   op(ctx, device)
 }
 
 /// Connect to a Nitrokey Storage device and do something with it.
-fn with_storage_device<F>(ctx: &mut ExecCtx<'_>, op: F) -> anyhow::Result<()>
+fn with_storage_device<F>(ctx: &mut Context<'_>, op: F) -> anyhow::Result<()>
 where
-  F: FnOnce(&mut ExecCtx<'_>, nitrokey::Storage<'_>) -> anyhow::Result<()>,
+  F: FnOnce(&mut Context<'_>, nitrokey::Storage<'_>) -> anyhow::Result<()>,
 {
   let mut manager =
     nitrokey::take().context("Failed to acquire access to Nitrokey device manager")?;
 
   set_log_level(ctx);
 
-  if let Some(model) = ctx.model {
+  if let Some(model) = ctx.config.model {
     if model != args::DeviceModel::Storage {
       anyhow::bail!("This command is only available on the Nitrokey Storage");
     }
+  } else {
+    ctx.config.model = Some(args::DeviceModel::Storage);
   }
 
-  let device = manager
-    .connect_storage()
-    .context("Nitrokey Storage device not found")?;
-  op(ctx, device)
+  let device = connect(&mut manager, &ctx.config)?;
+  if let nitrokey::DeviceWrapper::Storage(storage) = device {
+    op(ctx, storage)
+  } else {
+    panic!("connect returned a wrong model: {}", device.get_model())
+  }
 }
 
 /// Connect to any Nitrokey device, retrieve a password safe handle, and
 /// do something with it.
-fn with_password_safe<F>(ctx: &mut ExecCtx<'_>, mut op: F) -> anyhow::Result<()>
+fn with_password_safe<F>(ctx: &mut Context<'_>, mut op: F) -> anyhow::Result<()>
 where
-  F: FnMut(&mut ExecCtx<'_>, nitrokey::PasswordSafe<'_, '_>) -> anyhow::Result<()>,
+  F: FnMut(&mut Context<'_>, nitrokey::PasswordSafe<'_, '_>) -> anyhow::Result<()>,
 {
   with_device(ctx, |ctx, mut device| {
     let pin_entry = pinentry::PinEntry::from(args::PinType::User, &device)?;
@@ -118,14 +173,14 @@ where
 
 /// Authenticate the given device using the given PIN type and operation.
 fn authenticate<'mgr, D, A, F>(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   device: D,
   pin_type: args::PinType,
   op: F,
 ) -> anyhow::Result<A>
 where
   D: Device<'mgr>,
-  F: FnMut(&mut ExecCtx<'_>, D, &str) -> Result<A, (D, anyhow::Error)>,
+  F: FnMut(&mut Context<'_>, D, &str) -> Result<A, (D, anyhow::Error)>,
 {
   let pin_entry = pinentry::PinEntry::from(pin_type, &device)?;
 
@@ -134,7 +189,7 @@ where
 
 /// Authenticate the given device with the user PIN.
 fn authenticate_user<'mgr, T>(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   device: T,
 ) -> anyhow::Result<nitrokey::User<'mgr, T>>
 where
@@ -151,7 +206,7 @@ where
 
 /// Authenticate the given device with the admin PIN.
 fn authenticate_admin<'mgr, T>(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   device: T,
 ) -> anyhow::Result<nitrokey::Admin<'mgr, T>>
 where
@@ -194,13 +249,13 @@ fn get_volume_status(status: &nitrokey::VolumeStatus) -> &'static str {
 /// second or third try, it will call `op` with the data returned by the
 /// previous call to `op`.
 fn try_with_pin_and_data_with_pinentry<D, F, R>(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   pin_entry: &pinentry::PinEntry,
   data: D,
   mut op: F,
 ) -> anyhow::Result<R>
 where
-  F: FnMut(&mut ExecCtx<'_>, D, &str) -> Result<R, (D, anyhow::Error)>,
+  F: FnMut(&mut Context<'_>, D, &str) -> Result<R, (D, anyhow::Error)>,
 {
   let mut data = data;
   let mut retry = 3;
@@ -232,17 +287,17 @@ where
 
 /// Try to execute the given function with a PIN.
 fn try_with_pin_and_data<D, F, R>(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   pin_entry: &pinentry::PinEntry,
   data: D,
   mut op: F,
 ) -> anyhow::Result<R>
 where
-  F: FnMut(&mut ExecCtx<'_>, D, &str) -> Result<R, (D, anyhow::Error)>,
+  F: FnMut(&mut Context<'_>, D, &str) -> Result<R, (D, anyhow::Error)>,
 {
   let pin = match pin_entry.pin_type() {
     // Ideally we would not clone here, but that would require us to
-    // restrict op to work with an immutable ExecCtx, which is not
+    // restrict op to work with an immutable Context, which is not
     // possible given that some clients print data.
     args::PinType::Admin => ctx.admin_pin.clone(),
     args::PinType::User => ctx.user_pin.clone(),
@@ -251,7 +306,7 @@ where
   if let Some(pin) = pin {
     let pin = pin
       .to_str()
-      .ok_or_else(|| anyhow::anyhow!("Failed to read PIN: Invalid Unicode data found"))?;
+      .context("Failed to read PIN: Invalid Unicode data found")?;
     op(ctx, data, &pin).map_err(|(_, err)| err)
   } else {
     try_with_pin_and_data_with_pinentry(ctx, pin_entry, data, op)
@@ -263,7 +318,7 @@ where
 /// This function behaves exactly as `try_with_pin_and_data`, but
 /// it refrains from passing any data to it.
 fn try_with_pin<F>(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   pin_entry: &pinentry::PinEntry,
   mut op: F,
 ) -> anyhow::Result<()>
@@ -277,7 +332,7 @@ where
 
 /// Pretty print the status of a Nitrokey Storage.
 fn print_storage_status(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   status: &nitrokey::StorageStatus,
 ) -> anyhow::Result<()> {
   println!(
@@ -310,7 +365,7 @@ fn print_storage_status(
 
 /// Query and pretty print the status that is common to all Nitrokey devices.
 fn print_status(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   model: &'static str,
   device: &nitrokey::DeviceWrapper<'_>,
 ) -> anyhow::Result<()> {
@@ -351,7 +406,7 @@ fn print_status(
 }
 
 /// Inquire the status of the nitrokey.
-pub fn status(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
+pub fn status(ctx: &mut Context<'_>) -> anyhow::Result<()> {
   with_device(ctx, |ctx, device| {
     let model = match device {
       nitrokey::DeviceWrapper::Pro(_) => "Pro",
@@ -362,7 +417,7 @@ pub fn status(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
 }
 
 /// List the attached Nitrokey devices.
-pub fn list(ctx: &mut ExecCtx<'_>, no_connect: bool) -> anyhow::Result<()> {
+pub fn list(ctx: &mut Context<'_>, no_connect: bool) -> anyhow::Result<()> {
   set_log_level(ctx);
 
   let device_infos =
@@ -370,7 +425,7 @@ pub fn list(ctx: &mut ExecCtx<'_>, no_connect: bool) -> anyhow::Result<()> {
   if device_infos.is_empty() {
     println!(ctx, "No Nitrokey device connected")?;
   } else {
-    println!(ctx, "device path\tmodel\tserial number")?;
+    println!(ctx, "USB path\tmodel\tserial number")?;
     let mut manager =
       nitrokey::take().context("Failed to acquire access to Nitrokey device manager")?;
 
@@ -407,7 +462,7 @@ pub fn list(ctx: &mut ExecCtx<'_>, no_connect: bool) -> anyhow::Result<()> {
 }
 
 /// Perform a factory reset.
-pub fn reset(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
+pub fn reset(ctx: &mut Context<'_>) -> anyhow::Result<()> {
   with_device(ctx, |ctx, mut device| {
     let pin_entry = pinentry::PinEntry::from(args::PinType::Admin, &device)?;
 
@@ -436,7 +491,7 @@ pub fn reset(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
 
 /// Change the configuration of the unencrypted volume.
 pub fn unencrypted_set(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   mode: args::UnencryptedVolumeMode,
 ) -> anyhow::Result<()> {
   with_storage_device(ctx, |ctx, mut device| {
@@ -459,7 +514,7 @@ pub fn unencrypted_set(
 }
 
 /// Open the encrypted volume on the Nitrokey.
-pub fn encrypted_open(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
+pub fn encrypted_open(ctx: &mut Context<'_>) -> anyhow::Result<()> {
   with_storage_device(ctx, |ctx, mut device| {
     let pin_entry = pinentry::PinEntry::from(args::PinType::User, &device)?;
 
@@ -476,7 +531,7 @@ pub fn encrypted_open(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
 }
 
 /// Close the previously opened encrypted volume.
-pub fn encrypted_close(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
+pub fn encrypted_close(ctx: &mut Context<'_>) -> anyhow::Result<()> {
   with_storage_device(ctx, |_ctx, mut device| {
     // Flush all filesystem caches to disk. We are mostly interested in
     // making sure that the encrypted volume on the Nitrokey we are
@@ -491,13 +546,13 @@ pub fn encrypted_close(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
 }
 
 /// Create a hidden volume.
-pub fn hidden_create(ctx: &mut ExecCtx<'_>, slot: u8, start: u8, end: u8) -> anyhow::Result<()> {
+pub fn hidden_create(ctx: &mut Context<'_>, slot: u8, start: u8, end: u8) -> anyhow::Result<()> {
   with_storage_device(ctx, |ctx, mut device| {
     let pwd_entry = pinentry::PwdEntry::from(&device)?;
     let pwd = if let Some(pwd) = &ctx.password {
       pwd
         .to_str()
-        .ok_or_else(|| anyhow::anyhow!("Failed to read password: invalid unicode data found"))
+        .context("Failed to read password: Invalid Unicode data found")
         .map(ToOwned::to_owned)
     } else {
       pinentry::choose(ctx, &pwd_entry).context("Failed to select new PIN")
@@ -510,13 +565,13 @@ pub fn hidden_create(ctx: &mut ExecCtx<'_>, slot: u8, start: u8, end: u8) -> any
 }
 
 /// Open a hidden volume.
-pub fn hidden_open(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
+pub fn hidden_open(ctx: &mut Context<'_>) -> anyhow::Result<()> {
   with_storage_device(ctx, |ctx, mut device| {
     let pwd_entry = pinentry::PwdEntry::from(&device)?;
     let pwd = if let Some(pwd) = &ctx.password {
       pwd
         .to_str()
-        .ok_or_else(|| anyhow::anyhow!("Failed to read password: Invalid Unicode data found"))
+        .context("Failed to read password: Invalid Unicode data found")
         .map(ToOwned::to_owned)
     } else {
       pinentry::inquire(ctx, &pwd_entry, pinentry::Mode::Query, None)
@@ -534,7 +589,7 @@ pub fn hidden_open(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
 }
 
 /// Close a previously opened hidden volume.
-pub fn hidden_close(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
+pub fn hidden_close(ctx: &mut Context<'_>) -> anyhow::Result<()> {
   with_storage_device(ctx, |_ctx, mut device| {
     unsafe { sync() };
 
@@ -553,7 +608,7 @@ fn format_option<T: fmt::Display>(option: Option<T>) -> String {
 }
 
 /// Read the Nitrokey configuration.
-pub fn config_get(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
+pub fn config_get(ctx: &mut Context<'_>) -> anyhow::Result<()> {
   with_device(ctx, |ctx, device| {
     let config = device.get_config().context("Failed to get configuration")?;
     println!(
@@ -573,7 +628,7 @@ pub fn config_get(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
 }
 
 /// Write the Nitrokey configuration.
-pub fn config_set(ctx: &mut ExecCtx<'_>, args: args::ConfigSetArgs) -> anyhow::Result<()> {
+pub fn config_set(ctx: &mut Context<'_>, args: args::ConfigSetArgs) -> anyhow::Result<()> {
   let numlock = args::ConfigOption::try_from(args.no_numlock, args.numlock, "numlock")
     .context("Failed to apply numlock configuration")?;
   let capslock = args::ConfigOption::try_from(args.no_capslock, args.capslock, "capslock")
@@ -606,7 +661,7 @@ pub fn config_set(ctx: &mut ExecCtx<'_>, args: args::ConfigSetArgs) -> anyhow::R
 }
 
 /// Lock the Nitrokey device.
-pub fn lock(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
+pub fn lock(ctx: &mut Context<'_>) -> anyhow::Result<()> {
   with_device(ctx, |_ctx, mut device| {
     device.lock().context("Failed to lock the device")
   })
@@ -632,7 +687,7 @@ fn get_unix_timestamp() -> anyhow::Result<u64> {
 
 /// Generate a one-time password on the Nitrokey device.
 pub fn otp_get(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   slot: u8,
   algorithm: args::OtpAlgorithm,
   time: Option<u64>,
@@ -689,11 +744,11 @@ fn prepare_ascii_secret(secret: &str) -> anyhow::Result<String> {
 fn prepare_base32_secret(secret: &str) -> anyhow::Result<String> {
   base32::decode(base32::Alphabet::RFC4648 { padding: false }, secret)
     .map(|vec| format_bytes(&vec))
-    .ok_or_else(|| anyhow::anyhow!("Failed to parse base32 secret"))
+    .context("Failed to parse base32 secret")
 }
 
 /// Configure a one-time password slot on the Nitrokey device.
-pub fn otp_set(ctx: &mut ExecCtx<'_>, mut args: args::OtpSetArgs) -> anyhow::Result<()> {
+pub fn otp_set(ctx: &mut Context<'_>, mut args: args::OtpSetArgs) -> anyhow::Result<()> {
   let mut data = nitrokey::OtpSlotData {
     number: args.slot,
     name: mem::take(&mut args.name),
@@ -733,7 +788,7 @@ pub fn otp_set(ctx: &mut ExecCtx<'_>, mut args: args::OtpSetArgs) -> anyhow::Res
 
 /// Clear an OTP slot.
 pub fn otp_clear(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   slot: u8,
   algorithm: args::OtpAlgorithm,
 ) -> anyhow::Result<()> {
@@ -749,7 +804,7 @@ pub fn otp_clear(
 }
 
 fn print_otp_status(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   algorithm: args::OtpAlgorithm,
   device: &nitrokey::DeviceWrapper<'_>,
   all: bool,
@@ -762,7 +817,7 @@ fn print_otp_status(
     };
     slot = slot
       .checked_add(1)
-      .ok_or_else(|| anyhow::anyhow!("Encountered integer overflow when iterating OTP slots"))?;
+      .context("Encountered integer overflow when iterating OTP slots")?;
     let name = match result {
       Ok(name) => name,
       Err(nitrokey::Error::LibraryError(nitrokey::LibraryError::InvalidSlot)) => return Ok(()),
@@ -780,7 +835,7 @@ fn print_otp_status(
 }
 
 /// Print the status of the OTP slots.
-pub fn otp_status(ctx: &mut ExecCtx<'_>, all: bool) -> anyhow::Result<()> {
+pub fn otp_status(ctx: &mut Context<'_>, all: bool) -> anyhow::Result<()> {
   with_device(ctx, |ctx, device| {
     println!(ctx, "alg\tslot\tname")?;
     print_otp_status(ctx, args::OtpAlgorithm::Hotp, &device, all)?;
@@ -790,7 +845,7 @@ pub fn otp_status(ctx: &mut ExecCtx<'_>, all: bool) -> anyhow::Result<()> {
 }
 
 /// Clear the PIN stored by various operations.
-pub fn pin_clear(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
+pub fn pin_clear(ctx: &mut Context<'_>) -> anyhow::Result<()> {
   with_device(ctx, |_ctx, device| {
     pinentry::clear(&pinentry::PinEntry::from(args::PinType::Admin, &device)?)
       .context("Failed to clear admin PIN")?;
@@ -805,7 +860,7 @@ pub fn pin_clear(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
 /// If the user has set the respective environment variable for the
 /// given PIN type, it will be used.
 fn choose_pin(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   pin_entry: &pinentry::PinEntry,
   new: bool,
 ) -> anyhow::Result<String> {
@@ -829,7 +884,7 @@ fn choose_pin(
   if let Some(new_pin) = new_pin {
     new_pin
       .to_str()
-      .ok_or_else(|| anyhow::anyhow!("Failed to read PIN: Invalid Unicode data found"))
+      .context("Failed to read PIN: Invalid Unicode data found")
       .map(ToOwned::to_owned)
   } else {
     pinentry::choose(ctx, pin_entry).context("Failed to select PIN")
@@ -837,7 +892,7 @@ fn choose_pin(
 }
 
 /// Change a PIN.
-pub fn pin_set(ctx: &mut ExecCtx<'_>, pin_type: args::PinType) -> anyhow::Result<()> {
+pub fn pin_set(ctx: &mut Context<'_>, pin_type: args::PinType) -> anyhow::Result<()> {
   with_device(ctx, |ctx, mut device| {
     let pin_entry = pinentry::PinEntry::from(pin_type, &device)?;
     let new_pin = choose_pin(ctx, &pin_entry, true)?;
@@ -859,7 +914,7 @@ pub fn pin_set(ctx: &mut ExecCtx<'_>, pin_type: args::PinType) -> anyhow::Result
 }
 
 /// Unblock and reset the user PIN.
-pub fn pin_unblock(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
+pub fn pin_unblock(ctx: &mut Context<'_>) -> anyhow::Result<()> {
   with_device(ctx, |ctx, mut device| {
     let pin_entry = pinentry::PinEntry::from(args::PinType::User, &device)?;
     let user_pin = choose_pin(ctx, &pin_entry, false)?;
@@ -874,7 +929,7 @@ pub fn pin_unblock(ctx: &mut ExecCtx<'_>) -> anyhow::Result<()> {
 }
 
 fn print_pws_data(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   description: &'static str,
   result: Result<String, nitrokey::Error>,
   quiet: bool,
@@ -904,7 +959,7 @@ fn check_slot(pws: &nitrokey::PasswordSafe<'_, '_>, slot: u8) -> anyhow::Result<
 
 /// Read a PWS slot.
 pub fn pws_get(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   slot: u8,
   show_name: bool,
   show_login: bool,
@@ -930,7 +985,7 @@ pub fn pws_get(
 
 /// Write a PWS slot.
 pub fn pws_set(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   slot: u8,
   name: &str,
   login: &str,
@@ -944,14 +999,14 @@ pub fn pws_set(
 }
 
 /// Clear a PWS slot.
-pub fn pws_clear(ctx: &mut ExecCtx<'_>, slot: u8) -> anyhow::Result<()> {
+pub fn pws_clear(ctx: &mut Context<'_>, slot: u8) -> anyhow::Result<()> {
   with_password_safe(ctx, |_ctx, mut pws| {
     pws.erase_slot(slot).context("Failed to clear PWS slot")
   })
 }
 
 fn print_pws_slot(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   pws: &nitrokey::PasswordSafe<'_, '_>,
   slot: usize,
   programmed: bool,
@@ -969,7 +1024,7 @@ fn print_pws_slot(
 }
 
 /// Print the status of all PWS slots.
-pub fn pws_status(ctx: &mut ExecCtx<'_>, all: bool) -> anyhow::Result<()> {
+pub fn pws_status(ctx: &mut Context<'_>, all: bool) -> anyhow::Result<()> {
   with_password_safe(ctx, |ctx, pws| {
     let slots = pws
       .get_slot_status()
