@@ -1,21 +1,7 @@
 // main.rs
 
-// *************************************************************************
-// * Copyright (C) 2017-2020 Daniel Mueller (deso@posteo.net)              *
-// *                                                                       *
-// * This program is free software: you can redistribute it and/or modify  *
-// * it under the terms of the GNU General Public License as published by  *
-// * the Free Software Foundation, either version 3 of the License, or     *
-// * (at your option) any later version.                                   *
-// *                                                                       *
-// * This program is distributed in the hope that it will be useful,       *
-// * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-// * GNU General Public License for more details.                          *
-// *                                                                       *
-// * You should have received a copy of the GNU General Public License     *
-// * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
-// *************************************************************************
+// Copyright (C) 2017-2020 The Nitrocli Developers
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 #![warn(
   bad_style,
@@ -69,6 +55,7 @@ mod arg_util;
 
 mod args;
 mod commands;
+mod config;
 mod pinentry;
 #[cfg(test)]
 mod tests;
@@ -83,78 +70,15 @@ const NITROCLI_USER_PIN: &str = "NITROCLI_USER_PIN";
 const NITROCLI_NEW_ADMIN_PIN: &str = "NITROCLI_NEW_ADMIN_PIN";
 const NITROCLI_NEW_USER_PIN: &str = "NITROCLI_NEW_USER_PIN";
 const NITROCLI_PASSWORD: &str = "NITROCLI_PASSWORD";
-const NITROCLI_NO_CACHE: &str = "NITROCLI_NO_CACHE";
-
-trait Stdio {
-  fn stdio(&mut self) -> (&mut dyn io::Write, &mut dyn io::Write);
-}
-
-impl<'io> Stdio for RunCtx<'io> {
-  fn stdio(&mut self) -> (&mut dyn io::Write, &mut dyn io::Write) {
-    (self.stdout, self.stderr)
-  }
-}
-
-impl<W> Stdio for (&mut W, &mut W)
-where
-  W: io::Write,
-{
-  fn stdio(&mut self) -> (&mut dyn io::Write, &mut dyn io::Write) {
-    (self.0, self.1)
-  }
-}
-
-/// A command execution context that captures additional data pertaining
-/// the command execution.
-#[allow(missing_debug_implementations)]
-pub struct ExecCtx<'io> {
-  /// The Nitrokey model to use.
-  pub model: Option<args::DeviceModel>,
-  /// See `RunCtx::stdout`.
-  pub stdout: &'io mut dyn io::Write,
-  /// See `RunCtx::stderr`.
-  pub stderr: &'io mut dyn io::Write,
-  /// See `RunCtx::admin_pin`.
-  pub admin_pin: Option<ffi::OsString>,
-  /// See `RunCtx::user_pin`.
-  pub user_pin: Option<ffi::OsString>,
-  /// See `RunCtx::new_admin_pin`.
-  pub new_admin_pin: Option<ffi::OsString>,
-  /// See `RunCtx::new_user_pin`.
-  pub new_user_pin: Option<ffi::OsString>,
-  /// See `RunCtx::password`.
-  pub password: Option<ffi::OsString>,
-  /// See `RunCtx::no_cache`.
-  pub no_cache: bool,
-  /// The verbosity level to use for logging.
-  pub verbosity: u64,
-}
-
-impl<'io> Stdio for ExecCtx<'io> {
-  fn stdio(&mut self) -> (&mut dyn io::Write, &mut dyn io::Write) {
-    (self.stdout, self.stderr)
-  }
-}
 
 /// Parse the command-line arguments and execute the selected command.
-fn handle_arguments(ctx: &mut RunCtx<'_>, args: Vec<String>) -> anyhow::Result<()> {
+fn handle_arguments(ctx: &mut Context<'_>, args: Vec<String>) -> anyhow::Result<()> {
   use structopt::StructOpt;
 
   match args::Args::from_iter_safe(args.iter()) {
     Ok(args) => {
-      let mut ctx = ExecCtx {
-        model: args.model,
-        stdout: ctx.stdout,
-        stderr: ctx.stderr,
-        admin_pin: ctx.admin_pin.take(),
-        user_pin: ctx.user_pin.take(),
-        new_admin_pin: ctx.new_admin_pin.take(),
-        new_user_pin: ctx.new_user_pin.take(),
-        password: ctx.password.take(),
-        no_cache: ctx.no_cache,
-        verbosity: args.verbose.into(),
-      };
-      args.cmd.execute(&mut ctx)
+      ctx.config.update(&args);
+      args.cmd.execute(ctx)
     }
     Err(err) => {
       if err.use_stderr() {
@@ -168,7 +92,8 @@ fn handle_arguments(ctx: &mut RunCtx<'_>, args: Vec<String>) -> anyhow::Result<(
 }
 
 /// The context used when running the program.
-pub(crate) struct RunCtx<'io> {
+#[allow(missing_debug_implementations)]
+pub struct Context<'io> {
   /// The `Write` object used as standard output throughout the program.
   pub stdout: &'io mut dyn io::Write,
   /// The `Write` object used as standard error throughout the program.
@@ -187,11 +112,31 @@ pub(crate) struct RunCtx<'io> {
   pub new_user_pin: Option<ffi::OsString>,
   /// A password used by some commands, if provided through an environment variable.
   pub password: Option<ffi::OsString>,
-  /// Whether to bypass the cache for all secrets or not.
-  pub no_cache: bool,
+  /// The configuration, usually read from configuration files and environment
+  /// variables.
+  pub config: config::Config,
 }
 
-fn run<'ctx, 'io: 'ctx>(ctx: &'ctx mut RunCtx<'io>, args: Vec<String>) -> i32 {
+impl<'io> Context<'io> {
+  fn from_env<O, E>(stdout: &'io mut O, stderr: &'io mut E, config: config::Config) -> Context<'io>
+  where
+    O: io::Write,
+    E: io::Write,
+  {
+    Context {
+      stdout,
+      stderr,
+      admin_pin: env::var_os(NITROCLI_ADMIN_PIN),
+      user_pin: env::var_os(NITROCLI_USER_PIN),
+      new_admin_pin: env::var_os(NITROCLI_NEW_ADMIN_PIN),
+      new_user_pin: env::var_os(NITROCLI_NEW_USER_PIN),
+      password: env::var_os(NITROCLI_PASSWORD),
+      config,
+    }
+  }
+}
+
+fn run<'ctx, 'io: 'ctx>(ctx: &'ctx mut Context<'io>, args: Vec<String>) -> i32 {
   match handle_arguments(ctx, args) {
     Ok(()) => 0,
     Err(err) => {
@@ -206,19 +151,20 @@ fn main() {
 
   let mut stdout = io::stdout();
   let mut stderr = io::stderr();
-  let args = env::args().collect::<Vec<_>>();
-  let ctx = &mut RunCtx {
-    stdout: &mut stdout,
-    stderr: &mut stderr,
-    admin_pin: env::var_os(NITROCLI_ADMIN_PIN),
-    user_pin: env::var_os(NITROCLI_USER_PIN),
-    new_admin_pin: env::var_os(NITROCLI_NEW_ADMIN_PIN),
-    new_user_pin: env::var_os(NITROCLI_NEW_USER_PIN),
-    password: env::var_os(NITROCLI_PASSWORD),
-    no_cache: env::var_os(NITROCLI_NO_CACHE).is_some(),
+
+  let rc = match config::Config::load() {
+    Ok(config) => {
+      let args = env::args().collect::<Vec<_>>();
+      let ctx = &mut Context::from_env(&mut stdout, &mut stderr, config);
+
+      run(ctx, args)
+    }
+    Err(err) => {
+      let _ = writeln!(stderr, "{:?}", err);
+      1
+    }
   };
 
-  let rc = run(ctx, args);
   // We exit the process the hard way below. The problem is that because
   // of this, buffered IO may not be flushed. So make sure to explicitly
   // flush before exiting. Note that stderr is unbuffered, alleviating
