@@ -1,21 +1,7 @@
 // pinentry.rs
 
-// *************************************************************************
-// * Copyright (C) 2017-2020 Daniel Mueller (deso@posteo.net)              *
-// *                                                                       *
-// * This program is free software: you can redistribute it and/or modify  *
-// * it under the terms of the GNU General Public License as published by  *
-// * the Free Software Foundation, either version 3 of the License, or     *
-// * (at your option) any later version.                                   *
-// *                                                                       *
-// * This program is distributed in the hope that it will be useful,       *
-// * but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-// * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-// * GNU General Public License for more details.                          *
-// *                                                                       *
-// * You should have received a copy of the GNU General Public License     *
-// * along with this program.  If not, see <http://www.gnu.org/licenses/>. *
-// *************************************************************************
+// Copyright (C) 2017-2020 The Nitrocli Developers
+// SPDX-License-Identifier: GPL-3.0-or-later
 
 use std::borrow;
 use std::fmt;
@@ -25,7 +11,7 @@ use std::str;
 use anyhow::Context as _;
 
 use crate::args;
-use crate::ExecCtx;
+use crate::Context;
 
 type CowStr = borrow::Cow<'static, str>;
 
@@ -228,7 +214,7 @@ where
 /// dialog. It is used to choose an appropriate description and to
 /// decide whether a quality bar is shown in the dialog.
 pub fn inquire<E>(
-  ctx: &mut ExecCtx<'_>,
+  ctx: &mut Context<'_>,
   entry: &E,
   mode: Mode,
   error_msg: Option<&str>,
@@ -238,7 +224,7 @@ where
 {
   let cache_id = entry
     .cache_id()
-    .and_then(|id| if ctx.no_cache { None } else { Some(id) })
+    .and_then(|id| if ctx.config.no_cache { None } else { Some(id) })
     // "X" is a sentinel value indicating that no caching is desired.
     .unwrap_or_else(|| "X".into())
     .into();
@@ -283,7 +269,7 @@ where
   }
 }
 
-pub fn choose<E>(ctx: &mut ExecCtx<'_>, entry: &E) -> anyhow::Result<String>
+pub fn choose<E>(ctx: &mut Context<'_>, entry: &E) -> anyhow::Result<String>
 where
   E: SecretEntry,
 {
